
# 15-462 project makefile
# Authors: Eric Butler, Frank Palermo
# DO NOT EDIT THIS FILE

# srouce file diretory
SRC_DIR = src
# top-level output directory (for .o and .d files)
TOP_OBJ_DIR = bin
# the current directory
CURR_DIR = $(shell pwd)

TRG2D = p1_2d
TRGMODEL = p1_model
TRGCOASTER = p1_coaster

ifeq ($(TARGET),)
    SRCS +=
else ifeq ($(TARGET),$(TRG2D))
    SRCS += $(SRC2D)
else ifeq ($(TARGET),$(TRGMODEL))
    SRCS += $(SRCMODEL)
else ifeq ($(TARGET),$(TRGCOASTER))
    SRCS += $(SRCCOASTER)
else
    $(error "unknown target: $(TARGET)")
endif

# global compiler flags

CXX = g++
CXXFLAGS = -Wall -ansi -pedantic -I"$(CURR_DIR)/$(SRC_DIR)" -I"$(CURR_DIR)/include"
LDFLAGS = -L"./lib" -lGL -lGLU -lglut -lpng


# object directories, mode flags

# default to debug
ifeq ($(MODE),)
	MODE = debug
endif

ifeq ($(MODE), release)
	SUB_OBJ_DIR = release
	CXXFLAGS += -O2
else ifeq ($(MODE), debug)
	SUB_OBJ_DIR = debug
	CXXFLAGS += -g -O0
else
	$(error "unknown build mode: $(MODE)")
endif

OBJ_DIR = $(TOP_OBJ_DIR)/$(SUB_OBJ_DIR)

# list of all object files
OBJS = $(SRCS:.cpp=.o)
# list of all dep files
DEPS = $(OBJS:.o=.d)
# full list of paths to all objs
OBJS_FULL = $(addprefix $(OBJ_DIR)/,$(OBJS))
# full list of paths to all deps
DEPS_FULL = $(addprefix $(OBJ_DIR)/,$(DEPS))


# check for '.cpp' suffix to avoid accidentally blasting over source files
TMP_SRCS_NOT_CPP = $(filter-out %.cpp,$(SRCS))
ifneq (,$(TMP_SRCS_NOT_CPP))
	$(error "Feeling nervous about '$(TMP_SRCS_NOT_CPP)'; I only know how to \
		 build .cpp files")
endif



# targets

.PHONY: all release debug clean 

all:
	$(MAKE) $(MAKEFILE) MODE=$(MODE) TARGET=$(TRG2D) target
	$(MAKE) $(MAKEFILE) MODE=$(MODE) TARGET=$(TRGMODEL) target
	$(MAKE) $(MAKEFILE) MODE=$(MODE) TARGET=$(TRGCOASTER) target

target: $(OBJ_DIR)/$(TARGET)
	cp $< $(TARGET)

debug:
	$(MAKE) $(MAKEFILE) MODE=debug

release:
	$(MAKE) $(MAKEFILE) MODE=release

$(OBJ_DIR)/%.d: $(SRC_DIR)/%.cpp
	mkdir -p $(@D)
	$(CXX) $(CXXFLAGS) -MM -MP -MT $(@:.d=.o) -o $@ $<

# don't need to mkdir for object files since d files already exist
$(OBJ_DIR)/%.o: $(SRC_DIR)/%.cpp
	$(CXX) $(CXXFLAGS) -c -o $@ $<

ifneq ($(MAKECMDGOALS),clean)
-include $(DEPS_FULL)
endif

$(OBJ_DIR)/$(TARGET): $(OBJS_FULL)
	$(CXX) $(CXXFLAGS) $(LDFLAGS) -o $@ $^

clean:
	rm -rf $(TOP_OBJ_DIR) $(TRG2D) $(TRGMODEL) $(TRGCOASTER)

