/**
 * @file model_animation.cpp
 * @brief Staff animations for project 1 model.
 *
 * @author Eric Butler (edbutler)
 */

#include "application/model_animation.hpp"

/*
    DO NOT EDIT THIS FILE. The staff may supply updates to this file.
*/

namespace _462 {

#define NUM_SPLINE_SEGMENTS 16

void animate_control_points( std::string animation, real_t time, HermiteControlPointList& points )
{
    /*
        Here are sample animations. If you want to add new ones,
        DO NOT put them in this file, since staff may update this.
        The animation can be passed in as a commnand line argument,
        which will change the string passed to the initialize function.
        No argument results in the string "default" being passed.
    */

    static const std::string ANIM_DEFAULT_STR = "default";
    static const std::string ANIM_SIMPLE_STR = "simple";
    static const std::string ANIM_STATIC_STR = "static";

    // clear old points
    points.clear();

    if ( animation == ANIM_DEFAULT_STR ) {

        // a basic slithering animation

        static float yvals[NUM_SPLINE_SEGMENTS+1][2] = {
            {  .4,  .1 }, {  .3, -.25 }, {   0,   0 }, {   0,   0 },
            {   0,   0 }, {   0,   0  }, {   0,   0 }, {   0,   0 },
            {   0,   0 }
        };

        for ( size_t i = 0; i <= NUM_SPLINE_SEGMENTS; ++i ) {
            HermiteControlPoint cp;

            real_t t  = i;
            static const real_t dt = 1.0;
            static const real_t a = .03125;
            static const real_t b = .25 * PI;
            static const real_t c = 4.0 / NUM_SPLINE_SEGMENTS;
            static const real_t f = .0625;

            real_t ot = t - time * 6;

            cp.position   = Vector3( c *  t,  yvals[i][0], ( f + a * t ) * sin( b * ot )                              );
            cp.derivative = Vector3( c * dt,  yvals[i][1], ( f + a * t ) * cos( b * ot ) * b * dt + a * sin( b * ot ) );

            cp.up = Vector3::UnitY;
            Vector3 z = cp.derivative.cross( cp.up );
            cp.up = z.cross( cp.derivative );

            points.push_back( cp );
        }

    } else if ( animation == ANIM_SIMPLE_STR ) {

        // this animation doesn't have changes in y so up vectors are very simple

        for ( size_t i = 0; i <= NUM_SPLINE_SEGMENTS; ++i ) {
            HermiteControlPoint cp;

            real_t t  = i;
            static const real_t dt = 1.0;
            static const real_t a = .0125;
            static const real_t b = .25 * PI;
            static const real_t c = 4.0 / NUM_SPLINE_SEGMENTS;
            static const real_t f = .0625;

            real_t ot = t - time * 6;

            cp.position   = Vector3( c *  t,  0.0, ( f + a * t ) * sin( b * ot )                              );
            cp.derivative = Vector3( c * dt,  0.0, ( f + a * t ) * cos( b * ot ) * b * dt + a * sin( b * ot ) );
            cp.up = Vector3::UnitY;

            points.push_back( cp );
        }

    } else if ( animation == ANIM_STATIC_STR ) {

        // this animation doesn't change so it easier to test a single frame

        for ( size_t i = 0; i <= NUM_SPLINE_SEGMENTS; ++i ) {
            HermiteControlPoint cp;

            real_t t  = i;
            static const real_t dt = 1.0;
            static const real_t a = .0125;
            static const real_t b = .25 * PI;
            static const real_t c = 4.0 / NUM_SPLINE_SEGMENTS;
            static const real_t f = .0625;

            cp.position   = Vector3( c *  t,  0.0, ( f + a * t ) * sin( b * t )                             );
            cp.derivative = Vector3( c * dt,  0.0, ( f + a * t ) * cos( b * t ) * b * dt + a * sin( b * t ) );
            cp.up = Vector3::UnitY;

            points.push_back( cp );
        }
    }
}

} /* _462 */

