/**
 * @file spline_parser.hpp
 * @brief Functions for parsing a file of spline control points.
 *
 * @author Linus Li (zeyangl)
 */

#include "application/spline_parser.hpp"

#include <iostream>
#include <fstream>
#include <cstring>
#include <vector>
#include <sstream>

namespace _462 {

bool parse_spline_file( const char* filename, CatmullRomControlPointList& list, real_t* tau )
{
    std::ifstream file(filename);

    std::string line;

    if( !file.is_open() ){
        std::cout << "Cannot open spline file " << filename << std::endl;
        return false;
    }

    int line_num = 0;


    while( getline( file, line ) )
    {
        line_num++;

        std::stringstream stream( line );
        std::vector<float> line_tokens;

        while( !stream.fail() )
        {
            float token;
            stream >> token;
            line_tokens.push_back(token);
        }

          // remove end of line
        line_tokens.pop_back();

        if( line_num == 1)
        {
            if( line_tokens.size() != 1){
                std::cout << "Cannot find tension(tau) value" << std::endl;
                exit(1);
            }
            *tau = line_tokens[0];
            continue;
        }

        if( line_tokens.size() != 6 && line_tokens.size() != 0 && line_tokens.size() != 3)
        {
            printf("line %d has %lu tokens: \n", line_num, (unsigned long)line_tokens.size() );
            printf("incorrent number of elements specified, line ignored\n");
        }

        if( line_tokens.size() == 6 )
        {
            CatmullRomControlPoint point;
            point.position = Vector3( line_tokens[0], line_tokens[1], line_tokens[2] );
            point.up = Vector3( line_tokens[3], line_tokens[4], line_tokens[5] ).normalize();
            list.push_back(point);
        }else if( line_tokens.size() == 3 ){
             CatmullRomControlPoint point;
            point.position = Vector3( line_tokens[0], line_tokens[1], line_tokens[2] );
            point.up = Vector3(0, 1, 0).normalize();
            list.push_back(point);
        }
     }

    return true;
}

} /* _462 */

