/**
 * @file p1_model.cpp
 * @brief Project 1 implementation of spline-deformed model.
 *
 * @author Your Name (andrewid)
 * @bug Unimplemented.
 */

/*
    EDIT THIS FILE FOR P1.
 */

#include "p1_model.hpp"

#include "opengl.hpp"
#include "application/model_animation.hpp"
#include "scene/camera.hpp"
#include "scene/model.hpp"

#define NUM_DEFAULT_SPLINE_SEGMENTS 16

namespace _462 {

// the list of control points, updated each frame
static HermiteControlPointList control_points;
// the name of the animation to use
static std::string animation;

/**
 * Initializes the model for rendering with splines.
 * @param model The model to be animated.
 * @param animation_name The name of the animation to use.
 */
void p1_model_initialize( Model* model, const char* animation_name )
{
    glClearColor( 0.7f, 0.7f, 0.7f, 1.0f );

    // initialize spline segemnt list with defalut values
    for ( size_t i = 0; i <= NUM_DEFAULT_SPLINE_SEGMENTS; ++i ) {
        HermiteControlPoint cp;
        cp.position = Vector3::UnitX * (real_t(i) / 4.0);
        cp.derivative = Vector3::UnitX / 4.0;
        cp.up = Vector3::UnitY;
        control_points.push_back( cp );
    }

    // store the passed animation
    animation = animation_name;

    glEnable( GL_DEPTH_TEST );

    // TODO add your initialization code
}

/**
 * Renders the model using control_points.
 * @param camera The camera to be used for rendering the scene.
 */
void p1_model_render( const Camera& camera )
{
    // clear the buffers

    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
    glEnable( GL_DEPTH_TEST );

    // set transformation matrices based on camera

    glMatrixMode( GL_PROJECTION );
    glLoadIdentity();
    gluPerspective( camera.get_fov_degrees(),
                    camera.get_aspect_ratio(),
                    camera.get_near_clip(),
                    camera.get_far_clip() );

    glMatrixMode( GL_MODELVIEW );
    glLoadIdentity();

    const Vector3& campos = camera.get_position();
    const Vector3 camref = camera.get_direction() + campos;
    const Vector3 camup = camera.get_up();

    gluLookAt( campos.x, campos.y, campos.z,
               camref.x, camref.y, camref.z,
               camup.x,  camup.y,  camup.z );

    // TODO render the model using control_points as Hermite splines


    // renders a "compass" at the origin. you may find this extremely useful
    // when implementing your code, so we suggest leaving it in at first

    glColor3f(1.0, 0.0, 0.0);
    glBegin(GL_TRIANGLE_FAN);
    glVertex3d(  1.0,  0.0,  0.0 );
    glVertex3d(  0.1,  0.1,  0.1 );
    glVertex3d(  0.1, -0.1,  0.1 );
    glVertex3d(  0.1, -0.1, -0.1 );
    glVertex3d(  0.1,  0.1, -0.1 );
    glVertex3d(  0.1,  0.1,  0.1 );
    glEnd();

    glColor3f(0.0, 1.0, 0.0);
    glBegin(GL_TRIANGLE_FAN);
    glVertex3d(  0.0,  1.0,  0.0 );
    glVertex3d(  0.1,  0.1,  0.1 );
    glVertex3d( -0.1,  0.1,  0.1 );
    glVertex3d( -0.1,  0.1, -0.1 );
    glVertex3d(  0.1,  0.1, -0.1 );
    glVertex3d(  0.1,  0.1,  0.1 );
    glEnd();

    glColor3f(0.0, 0.0, 1.0);
    glBegin(GL_TRIANGLE_FAN);
    glVertex3d(  0.0,  0.0,  1.0 );
    glVertex3d(  0.1,  0.1,  0.1 );
    glVertex3d( -0.1,  0.1,  0.1 );
    glVertex3d( -0.1, -0.1,  0.1 );
    glVertex3d(  0.1, -0.1,  0.1 );
    glVertex3d(  0.1,  0.1,  0.1 );
    glEnd();
}

/**
 * Tickes the current frame. This function should update
 * the list of control points to be rendered.
 * @param delta_time The change in time from the previous frame.
 */
void p1_model_update( real_t delta_time )
{
    static real_t time = 0;
    time += delta_time;

    // check to see if it's a staff animation; if so, animate it
    animate_control_points( animation, time, control_points ); 

    // TODO add you own animations if you like
}


} /* _462 */

