/**
 * @file model.hpp
 * @brief Model class
 *
 * @author Eric Butler (edbutler)
 */

#ifndef _462_SCENE_MODEL_HPP_
#define _462_SCENE_MODEL_HPP_

#include "math/vector.hpp"
#include "math/quaternion.hpp"
#include <vector>
#include <cassert>

namespace _462 {


/**
 * A mesh of triangles.
 */
class Model
{
public:

    struct Vertex
    {
        Vector3 position;
        Vector3 normal;
        Vector2 tex_coord;
    };

    struct Triangle
    {
        // index into the vertex list of the 3 vertices
        size_t vertices[3];
    };

    typedef std::vector< Triangle > TriangleList;
    typedef std::vector< Vertex > VertexList;

    const VertexList& get_vertices() const
    {
        return vertices;
    }

    const TriangleList& get_triangles() const
    {
        return triangles;
    }

    /**
     * @return true on success, false on failure.
     */
    bool load_from_obj( const char* filename );

    // The filename of the texture to use with this model.
    std::string texture_filename;

protected:

    // The list of all triangles in this model.
    TriangleList triangles;
    // The list of all vertices in this model.
    VertexList vertices;

};


} /* _462 */

#endif /* _462_SCENE_MODEL_HPP_ */

